@@ -85,11 +85,9 @@
 #      |___________|         |___> x
 #      0    e3     1
 
-import properties
-
 from discretize.base import BaseTensorMesh
 from discretize.operators import InnerProducts, DiffOperators
-from discretize.base.mesh_io import TreeMeshIO
+from discretize.mixins import InterfaceMixins, TreeMeshIO
 from discretize.utils import as_array_n_by_dim
 from discretize._extensions.tree_ext import _TreeMesh, TreeCell
 import numpy as np
@@ -98,9 +96,72 @@ import warnings
 from discretize.utils.code_utils import deprecate_property
 
 
-class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
-    """
-    TreeMesh is a class for adaptive QuadTree (2D) and OcTree (3D) meshes.
+class TreeMesh(
+    _TreeMesh, InnerProducts, DiffOperators, BaseTensorMesh, TreeMeshIO, InterfaceMixins,
+):
+    """Class for QuadTree (2D) and OcTree (3D) meshes.
+
+    Tree meshes are numerical grids where the dimensions of each cell are powers of 2
+    larger than some base cell dimension. Unlike the :class:`~discretize.TensorMesh`
+    class, gridded locations and numerical operators for instances of ``TreeMesh``
+    cannot be simply constructed using tensor products. Furthermore, each cell
+    is an instance of ``TreeMesh`` is an instance of the
+    :class:`~discretize.tree_mesh.TreeCell` .
+
+    Parameters
+    ----------
+    h : (dim) iterable of int, numpy.ndarray, or tuple
+        Defines the cell widths of the *underlying tensor mesh* along each axis. The
+        length of the iterable object is equal to the dimension of the mesh (2 or 3).
+        For a 3D mesh, the list would have the form *[hx, hy, hz]*. The number of cells
+        along each axis **must be a power of 2** .
+
+        Along each axis, the user has 3 choices for defining the cells widths for the
+        underlying tensor mesh:
+
+        - :class:`int` -> A unit interval is equally discretized into `N` cells.
+        - :class:`numpy.ndarray` -> The widths are explicity given for each cell
+        - the widths are defined as a :class:`list` of :class:`tuple` of the form *(dh, nc, [npad])*
+          where *dh* is the cell width, *nc* is the number of cells, and *npad* (optional)
+          is a padding factor denoting exponential increase/decrease in the cell width
+          for each cell; e.g. *[(2., 10, -1.3), (2., 50), (2., 10, 1.3)]*
+
+    origin : (dim) iterable, default: 0
+        Define the origin or 'anchor point' of the mesh; i.e. the bottom-left-frontmost
+        corner. By default, the mesh is anchored such that its origin is at [0, 0, 0].
+
+        For each dimension (x, y or z), The user may set the origin 2 ways:
+
+        - a ``scalar`` which explicitly defines origin along that dimension.
+        - **{'0', 'C', 'N'}** a :class:`str` specifying whether the zero coordinate along
+          each axis is the first node location ('0'), in the center ('C') or the last
+          node location ('N') (see Examples).
+
+    Examples
+    --------
+    Here we generate a basic 2D tree mesh.
+
+    >>> from discretize import TreeMesh
+    >>> import numpy as np
+    >>> import matplotlib.pyplot as plt
+
+    Define base mesh (domain and finest discretization),
+
+    >>> dh = 5    # minimum cell width (base mesh cell width)
+    >>> nbc = 64  # number of base mesh cells
+    >>> h = dh * np.ones(nbc)
+    >>> mesh = TreeMesh([h, h])
+
+    Define corner points for a rectangular box, and subdived the mesh within the box
+    to the maximum refinement level.
+
+    >>> x0s = [120.0, 80.0]
+    >>> x1s = [240.0, 160.0]
+    >>> levels = [mesh.max_level]
+    >>> mesh.refine_box(x0s, x1s, levels)
+
+    >>> mesh.plot_grid()
+    >>> plt.show()
     """
 
     _meshType = "TREE"
@@ -135,31 +196,26 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
             "gridhEz": "hanging_edges_z",
         },
     }
+    _items = {"h", "origin", "cell_state"}
 
     # inheriting stuff from BaseTensorMesh that isn't defined in _QuadTree
     def __init__(self, h=None, origin=None, **kwargs):
         if "x0" in kwargs:
             origin = kwargs.pop("x0")
-        BaseTensorMesh.__init__(
-            self, h, origin
-        )  # TODO:, **kwargs) # pass the kwargs for copy/paste
+        super().__init__(h=h, origin=origin)
 
-        nx = len(self.h[0])
-        ny = len(self.h[1])
-        nz = len(self.h[2]) if self.dim == 3 else 2
-
-        def is_pow2(num):
-            return ((num & (num - 1)) == 0) and num != 0
-
-        if not (is_pow2(nx) and is_pow2(ny) and is_pow2(nz)):
-            raise ValueError("length of cell width vectors must be a power of 2")
-        # Now can initialize cpp tree parent
-        _TreeMesh.__init__(self, self.h, self.origin)
-
-        if "cell_levels" in kwargs.keys() and "cell_indexes" in kwargs.keys():
-            inds = kwargs.pop("cell_indexes")
-            levels = kwargs.pop("cell_levels")
-            self.__setstate__((inds, levels))
+        cell_state = kwargs.pop("cell_state", None)
+        cell_indexes = kwargs.pop("cell_indexes", None)
+        cell_levels = kwargs.pop("cell_levels", None)
+        if cell_state is None:
+            if cell_indexes is not None and cell_levels is not None:
+                cell_state = {}
+                cell_state["indexes"] = cell_indexes
+                cell_state["levels"] = cell_levels
+        if cell_state is not None:
+            indexes = cell_state["indexes"]
+            levels = cell_state["levels"]
+            self.__setstate__((indexes, levels))
 
     def __repr__(self):
         """Plain text representation."""
@@ -287,18 +343,43 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
         return full_tbl
 
-    @properties.validator("origin")
-    def _origin_validator(self, change):
-        self._set_origin(change["value"])
+    @BaseTensorMesh.origin.setter
+    def origin(self, value):
+        # first use the BaseTensorMesh to set the origin to handle "0, C, N"
+        BaseTensorMesh.origin.fset(self, value)
+        # then update the TreeMesh with the hidden value
+        self._set_origin(self._origin)
 
     @property
     def vntF(self):
-        """Total number of hanging and non-hanging faces in a [nx,ny,nz] form"""
+        """
+        Vector number of total faces along each axis
+
+        This property returns the total number of hanging and
+        non-hanging faces along each axis direction. The returned
+        quantity is a list of integers of the form [nFx,nFy,nFz].
+
+        Returns
+        -------
+        list of int
+            Vector number of total faces along each axis
+        """
         return [self.ntFx, self.ntFy] + ([] if self.dim == 2 else [self.ntFz])
 
     @property
     def vntE(self):
-        """Total number of hanging and non-hanging edges in a [nx,ny,nz] form"""
+        """
+        Vector number of total edges along each axis
+
+        This property returns the total number of hanging and
+        non-hanging edges along each axis direction. The returned
+        quantity is a list of integers of the form [nEx,nEy,nEz].
+
+        Returns
+        -------
+        list of int
+            Vector number of total edges along each axis
+        """
         return [self.ntEx, self.ntEy] + ([] if self.dim == 2 else [self.ntEz])
 
     @property
@@ -317,10 +398,6 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def cell_gradient(self):
-        """
-        Cell centered Gradient operator built off of the faceDiv operator.
-        Grad =  - (Mf)^{-1} * Div * diag (volume)
-        """
         if getattr(self, "_cell_gradient", None) is None:
 
             i_s = self.face_boundary_indices
@@ -355,10 +432,6 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def cell_gradient_x(self):
-        """
-        Cell centered Gradient operator in x-direction (Gradx)
-        Grad = sp.vstack((Gradx, Grady, Gradz))
-        """
         if getattr(self, "_cell_gradient_x", None) is None:
 
             nFx = self.nFx
@@ -380,10 +453,6 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def cell_gradient_y(self):
-        """
-        Cell centered Gradient operator in y-direction (Grady)
-        Grad = sp.vstack((Gradx, Grady, Gradz))
-        """
         if getattr(self, "_cell_gradient_y", None) is None:
 
             nFx = self.nFx
@@ -406,10 +475,6 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def cell_gradient_z(self):
-        """
-        Cell centered Gradient operator in z-direction (Gradz)
-        Grad = sp.vstack((Gradx, Grady, Gradz))
-        """
         if self.dim == 2:
             raise TypeError("z derivative not defined in 2D")
         if getattr(self, "_cell_gradient_z", None) is None:
@@ -441,7 +506,9 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
     @property
     def face_y_divergence(self):
         if getattr(self, "_face_y_divergence", None) is None:
-            self._face_y_divergence = self.face_divergence[:, self.nFx : self.nFx + self.nFy]
+            self._face_y_divergence = self.face_divergence[
+                :, self.nFx : self.nFx + self.nFy
+            ]
         return self._face_y_divergence
 
     @property
@@ -451,20 +518,6 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
         return self._face_z_divergence
 
     def point2index(self, locs):
-        """Finds cells that contain the given points.
-        Returns an array of index values of the cells that contain the given
-        points
-
-        Parameters
-        ----------
-        locs: array_like of shape (N, dim)
-            points to search for the location of
-
-        Returns
-        -------
-        numpy.array of integers of length(N)
-            Cell indices that contain the points
-        """
         locs = as_array_n_by_dim(locs, self.dim)
         inds = self._get_containing_cell_indexes(locs)
         return inds
@@ -474,12 +527,12 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
         Parameters
         ----------
-        index: array_like of length (N)
+        index: (N) array_like
             Cell indexes to query
 
         Returns
         -------
-        numpy.array of length (N)
+        (N) numpy.ndarray of int
             Levels for the cells.
         """
 
@@ -488,50 +541,25 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
     def get_interpolation_matrix(
         self, locs, location_type="CC", zeros_outside=False, **kwargs
     ):
-        """Produces interpolation matrix
-
-        Parameters
-        ----------
-        loc : numpy.ndarray
-            Location of points to interpolate to
-
-        location_type: str
-            What to interpolate
-
-            location_type can be::
-
-                'Ex'    -> x-component of field defined on edges
-                'Ey'    -> y-component of field defined on edges
-                'Ez'    -> z-component of field defined on edges
-                'Fx'    -> x-component of field defined on faces
-                'Fy'    -> y-component of field defined on faces
-                'Fz'    -> z-component of field defined on faces
-                'N'     -> scalar field defined on nodes
-                'CC'    -> scalar field defined on cell centers
-
-        Returns
-        -------
-        scipy.sparse.csr_matrix
-            M, the interpolation matrix
-
-        """
         if "locType" in kwargs:
             warnings.warn(
                 "The locType keyword argument has been deprecated, please use location_type. "
                 "This will be removed in discretize 1.0.0",
-                DeprecationWarning,
+                FutureWarning,
             )
             location_type = kwargs["locType"]
         if "zerosOutside" in kwargs:
             warnings.warn(
                 "The zerosOutside keyword argument has been deprecated, please use zeros_outside. "
                 "This will be removed in discretize 1.0.0",
-                DeprecationWarning,
+                FutureWarning,
             )
             zeros_outside = kwargs["zerosOutside"]
         locs = as_array_n_by_dim(locs, self.dim)
         if location_type not in ["N", "CC", "Ex", "Ey", "Ez", "Fx", "Fy", "Fz"]:
-            raise Exception("location_type must be one of N, CC, Ex, Ey, Ez, Fx, Fy, or Fz")
+            raise Exception(
+                "location_type must be one of N, CC, Ex, Ey, Ez, Fx, Fy, or Fz"
+            )
 
         if self.dim == 2 and location_type in ["Ez", "Fz"]:
             raise Exception("Unable to interpolate from Z edges/face in 2D")
@@ -550,14 +578,24 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def permute_cells(self):
-        """Permutation matrix re-ordering of cells sorted by x, then y, then z"""
+        """Permutation matrix re-ordering of cells sorted by x, then y, then z
+
+        Returns
+        -------
+        (n_cells, n_cells) scipy.sparse.csr_matrix
+        """
         # TODO: cache these?
         P = np.lexsort(self.gridCC.T)  # sort by x, then y, then z
         return sp.identity(self.nC).tocsr()[P]
 
     @property
     def permute_faces(self):
-        """Permutation matrix re-ordering of faces sorted by x, then y, then z"""
+        """Permutation matrix re-ordering of faces sorted by x, then y, then z
+
+        Returns
+        -------
+        (n_faces, n_faces) scipy.sparse.csr_matrix
+        """
         # TODO: cache these?
         Px = np.lexsort(self.gridFx.T)
         Py = np.lexsort(self.gridFy.T) + self.nFx
@@ -570,7 +608,12 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
 
     @property
     def permute_edges(self):
-        """Permutation matrix re-ordering of edges sorted by x, then y, then z"""
+        """Permutation matrix re-ordering of edges sorted by x, then y, then z
+
+        Returns
+        -------
+        (n_edges, n_edges) scipy.sparse.csr_matrix
+        """
         # TODO: cache these?
         Px = np.lexsort(self.gridEx.T)
         Py = np.lexsort(self.gridEy.T) + self.nEx
@@ -581,82 +624,96 @@ class TreeMesh(_TreeMesh, BaseTensorMesh, InnerProducts, TreeMeshIO):
             P = np.r_[Px, Py, Pz]
         return sp.identity(self.nE).tocsr()[P]
 
-    def serialize(self):
-        serial = BaseTensorMesh.serialize(self)
-        inds, levels = self.__getstate__()
-        serial["cell_indexes"] = inds.tolist()
-        serial["cell_levels"] = levels.tolist()
-        return serial
+    @property
+    def cell_state(self):
+        """ The current state of the cells on the mesh.
 
-    @classmethod
-    def deserialize(cls, serial):
-        mesh = cls(**serial)
-        return mesh
+        This represents the x, y, z indices of the cells in the base tensor mesh, as
+        well as their levels. It can be used to reconstruct the mesh.
+
+        Returns
+        -------
+        dict
+            dictionary with two entries:
+
+            - ``"indexes"``: the indexes of the cells
+            - ``"levels"``: the levels of the cells
+        """
+        indexes, levels = self.__getstate__()
+        return {"indexes": indexes.tolist(), "levels": levels.tolist()}
+
+    def validate(self):
+        return self.finalized
+
+    def equals(self, other):
+        try:
+            if self.finalized and other.finalized:
+                return super().equals(other)
+        except AttributeError:
+            pass
+        return False
 
     def __reduce__(self):
         return TreeMesh, (self.h, self.origin), self.__getstate__()
 
-    cellGrad = deprecate_property("cell_gradient", "cellGrad", removal_version="1.0.0")
+    cellGrad = deprecate_property("cell_gradient", "cellGrad", removal_version="1.0.0", future_warn=True)
     cellGradx = deprecate_property(
-        "cell_gradient_x", "cellGradx", removal_version="1.0.0"
+        "cell_gradient_x", "cellGradx", removal_version="1.0.0", future_warn=True
     )
     cellGrady = deprecate_property(
-        "cell_gradient_y", "cellGrady", removal_version="1.0.0"
+        "cell_gradient_y", "cellGrady", removal_version="1.0.0", future_warn=True
     )
     cellGradz = deprecate_property(
-        "cell_gradient_z", "cellGradz", removal_version="1.0.0"
+        "cell_gradient_z", "cellGradz", removal_version="1.0.0", future_warn=True
     )
     cellGradStencil = deprecate_property(
-        "cell_gradient_stencil", "cellGradStencil", removal_version="1.0.0"
+        "cell_gradient_stencil", "cellGradStencil", removal_version="1.0.0", future_warn=True
     )
-    nodalGrad = deprecate_property(
-        "nodal_gradient", "nodalGrad", removal_version="1.0.0"
-    )
-    nodalLaplacian = deprecate_property(
-        "nodal_laplacian", "nodalLaplacian", removal_version="1.0.0"
-    )
-    faceDiv = deprecate_property("face_divergence", "faceDiv", removal_version="1.0.0")
     faceDivx = deprecate_property(
-        "face_x_divergence", "faceDivx", removal_version="1.0.0"
+        "face_x_divergence", "faceDivx", removal_version="1.0.0", future_warn=True
     )
     faceDivy = deprecate_property(
-        "face_y_divergence", "faceDivy", removal_version="1.0.0"
+        "face_y_divergence", "faceDivy", removal_version="1.0.0", future_warn=True
     )
     faceDivz = deprecate_property(
-        "face_z_divergence", "faceDivz", removal_version="1.0.0"
+        "face_z_divergence", "faceDivz", removal_version="1.0.0", future_warn=True
     )
-    edgeCurl = deprecate_property("edge_curl", "edgeCurl", removal_version="1.0.0")
-    maxLevel = deprecate_property("max_used_level", "maxLevel", removal_version="1.0.0")
-    vol = deprecate_property("cell_volumes", "vol", removal_version="1.0.0")
-    areaFx = deprecate_property("face_x_areas", "areaFx", removal_version="1.0.0")
-    areaFy = deprecate_property("face_y_areas", "areaFy", removal_version="1.0.0")
-    areaFz = deprecate_property("face_z_areas", "areaFz", removal_version="1.0.0")
-    area = deprecate_property("face_areas", "area", removal_version="1.0.0")
-    edgeEx = deprecate_property("edge_x_lengths", "edgeEx", removal_version="1.0.0")
-    edgeEy = deprecate_property("edge_y_lengths", "edgeEy", removal_version="1.0.0")
-    edgeEz = deprecate_property("edge_z_lengths", "edgeEz", removal_version="1.0.0")
-    edge = deprecate_property("edge_lengths", "edge", removal_version="1.0.0")
+    maxLevel = deprecate_property("max_used_level", "maxLevel", removal_version="1.0.0", future_warn=True)
+    areaFx = deprecate_property("face_x_areas", "areaFx", removal_version="1.0.0", future_warn=True)
+    areaFy = deprecate_property("face_y_areas", "areaFy", removal_version="1.0.0", future_warn=True)
+    areaFz = deprecate_property("face_z_areas", "areaFz", removal_version="1.0.0", future_warn=True)
+    edgeEx = deprecate_property("edge_x_lengths", "edgeEx", removal_version="1.0.0", future_warn=True)
+    edgeEy = deprecate_property("edge_y_lengths", "edgeEy", removal_version="1.0.0", future_warn=True)
+    edgeEz = deprecate_property("edge_z_lengths", "edgeEz", removal_version="1.0.0", future_warn=True)
     permuteCC = deprecate_property(
-        "permute_cells", "permuteCC", removal_version="1.0.0"
+        "permute_cells", "permuteCC", removal_version="1.0.0", future_warn=True
     )
-    permuteF = deprecate_property("permute_faces", "permuteF", removal_version="1.0.0")
-    permuteE = deprecate_property("permute_edges", "permuteE", removal_version="1.0.0")
+    permuteF = deprecate_property("permute_faces", "permuteF", removal_version="1.0.0", future_warn=True)
+    permuteE = deprecate_property("permute_edges", "permuteE", removal_version="1.0.0", future_warn=True)
     faceBoundaryInd = deprecate_property(
-        "face_boundary_indices", "faceBoundaryInd", removal_version="1.0.0"
+        "face_boundary_indices", "faceBoundaryInd", removal_version="1.0.0", future_warn=True
     )
     cellBoundaryInd = deprecate_property(
-        "cell_boundary_indices", "cellBoundaryInd", removal_version="1.0.0"
+        "cell_boundary_indices", "cellBoundaryInd", removal_version="1.0.0", future_warn=True
     )
     _aveCC2FxStencil = deprecate_property(
-        "average_cell_to_total_face_x", "_aveCC2FxStencil", removal_version="1.0.0"
+        "average_cell_to_total_face_x", "_aveCC2FxStencil", removal_version="1.0.0", future_warn=True
     )
     _aveCC2FyStencil = deprecate_property(
-        "average_cell_to_total_face_y", "_aveCC2FyStencil", removal_version="1.0.0"
+        "average_cell_to_total_face_y", "_aveCC2FyStencil", removal_version="1.0.0", future_warn=True
     )
     _aveCC2FzStencil = deprecate_property(
-        "average_cell_to_total_face_z", "_aveCC2FzStencil", removal_version="1.0.0"
+        "average_cell_to_total_face_z", "_aveCC2FzStencil", removal_version="1.0.0", future_warn=True
+    )
+    _cellGradStencil = deprecate_property(
+        "stencil_cell_gradient", "_cellGradStencil", removal_version="1.0.0", future_warn=True
+    )
+    _cellGradxStencil = deprecate_property(
+        "stencil_cell_gradient_x", "_cellGradxStencil", removal_version="1.0.0", future_warn=True
+    )
+    _cellGradyStencil = deprecate_property(
+        "stencil_cell_gradient_y", "_cellGradyStencil", removal_version="1.0.0", future_warn=True
+    )
+    _cellGradzStencil = deprecate_property(
+        "stencil_cell_gradient_z", "_cellGradzStencil", removal_version="1.0.0", future_warn=True
     )
-    _cellGradStencil = deprecate_property("stencil_cell_gradient", "_cellGradStencil", removal_version="1.0.0")
-    _cellGradxStencil = deprecate_property("stencil_cell_gradient_x", "_cellGradxStencil", removal_version="1.0.0")
-    _cellGradyStencil = deprecate_property("stencil_cell_gradient_y", "_cellGradyStencil", removal_version="1.0.0")
-    _cellGradzStencil = deprecate_property("stencil_cell_gradient_z", "_cellGradzStencil", removal_version="1.0.0")
